def to_s(text):
    return text if isinstance(text, str) else text.decode("utf-8")

def get_env_for_shell(query):

    def join_options(opt):
        name = to_s(opt[0])
        values = ";".join([to_s(x).replace(';', '\\;') for x in opt[1]])
        return name, values

    return dict(list(map(join_options, list(query.items()))))



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2008 Wilson Pinto Júnior <wilson@openlanhouse.org>
#  Copyright (C) 2011 Thomaz de Oliveira dos Reis <thor27@gmail.com>
#  Copyright (C) 2021 Elton Fabrício Ferreira <eltonfabricio10@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os

APP_NAME = "BigBashView"
APP_VERSION = "3.6.6"
PROGDIR = os.path.dirname(os.path.realpath(__file__))
ICON = os.sep.join((PROGDIR, "img", "icone.png"))
LOGO = os.sep.join((PROGDIR, "img", "logo.png"))
TITLE = None
ROOT_FILE = False

def ADDRESS(): return '127.0.0.1'
def PORT(): return 9000



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2008 Wilson Pinto Júnior <wilson@openlanhouse.org>
#  Copyright (C) 2011 Thomaz de Oliveira dos Reis <thor27@gmail.com>
#  Copyright (C) 2009  Bruno Goncalves Araujo
#  Copyright (C) 2021 Elton Fabrício Ferreira <eltonfabricio10@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
import os
from PySide2.QtCore import QUrl, Qt
from PySide2.QtWidgets import QWidget, QHBoxLayout, QSplitter, QApplication, QShortcut
from PySide2.QtGui import QIcon, QColor, QKeySequence
from PySide2.QtWebEngineWidgets import QWebEngineView

from bbv.globaldata import ICON, TITLE

class Window(QWidget):
    def __init__(self):
        self.app = QApplication(sys.argv)
        super().__init__()
        self.web = QWebEngineView()
        self.inspector = QWebEngineView()
        self.web.settings().setAttribute(
                self.web.settings().AutoLoadIconsForPage, False)
        self.setWindowIcon(QIcon(ICON))
        if TITLE:
            self.app.setApplicationName(TITLE)
            self.setWindowTitle(TITLE)
        else:
            self.web.titleChanged.connect(self.title_changed)
        self.web.page().windowCloseRequested.connect(self.close_window)
        self.web.loadFinished.connect(self.add_script)
        self.key_f5 = QShortcut(QKeySequence(Qt.Key_F5), self.web)
        self.key_f5.activated.connect(self.web.reload)
        self.key_f12 = QShortcut(QKeySequence(Qt.Key_F12), self.web)
        self.key_f12.activated.connect(self.devpage)

        # pagesplitter
        self.hbox = QHBoxLayout(self)
        self.hbox.setContentsMargins(0,0,0,0)
        self.splitter = QSplitter(Qt.Horizontal)
        self.splitter.addWidget(self.web)
        self.splitter2 = QSplitter(Qt.Horizontal)
        self.hbox.addWidget(self.splitter)
        self.setLayout(self.hbox)

    def devpage(self):
        if self.splitter.count() == 1 or self.splitter2.count() == 1:
            self.inspector.page().setInspectedPage(self.web.page())
            self.splitter2.hide()
            self.splitter.addWidget(self.web)
            self.splitter.addWidget(self.inspector)
            self.splitter.setSizes([(self.width()/3)*2, self.width()/3])
            self.splitter.show()
            self.hbox.addWidget(self.splitter)
            self.setLayout(self.hbox)
        else:
            self.splitter.hide()
            self.splitter2.addWidget(self.web)
            self.splitter2.show()
            self.hbox.addWidget(self.splitter2)
            self.setLayout(self.hbox)

    def add_script(self, event):
        script = '''
        function _run(run){
            fetch("/execute$"+run);
        };
        '''
        if event:
            self.web.page().runJavaScript(script)

    def viewer(self, window_state):
        if window_state == "maximized":
            self.setWindowState(Qt.WindowMaximized)
            self.show()
        elif window_state == "fullscreen":
            self.setWindowState(Qt.WindowFullScreen)
            self.show()
        elif window_state == "maximizedTop":
            self.setWindowState(Qt.WindowMaximized)
            self.setWindowFlags(Qt.WindowStaysOnTopHint)
            self.show()
        elif window_state == "frameless":
            self.setWindowFlags(Qt.FramelessWindowHint)
            self.show()
        elif window_state == "framelessTop":
            self.setWindowFlags(Qt.FramelessWindowHint|Qt.WindowStaysOnTopHint)
            self.show()
        elif window_state == "fixedTop":
            self.setWindowFlags(Qt.WindowStaysOnTopHint)
            self.show()
        else:
            self.show()

    def run(self):
        self.app.exec_()

    def close_window(self):
        self.app.quit()

    def title_changed(self, title):
        os.system('''xprop -id "$(xprop -root '\t$0' _NET_ACTIVE_WINDOW | cut -f 2)" \
                    -f WM_CLASS 8s -set WM_CLASS "%s"''' % title)
        self.setWindowTitle(title)

    def load_url(self, url):
        self.url = QUrl.fromEncoded(url.encode("utf-8"))
        self.web.load(self.url)

    def set_size(self, width, height, window_state):
        display = self.app.primaryScreen()
        size = display.availableGeometry()
        if width <= 0:
            width = size.width()/2
        if height <= 0:
            height = size.height()/2

        self.resize(width, height)
        qr = self.frameGeometry()
        cp = display.availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())
        if window_state in ("fixed", "fixedTop"):
            self.setFixedSize(width, height)

    def style(self, colorful):
        if colorful == 'black':
            self.web.page().setBackgroundColor(QColor.fromRgbF(0, 0, 0, 1))

        elif colorful == 'none':
            self.setAttribute(Qt.WA_TranslucentBackground)
            self.web.page().setBackgroundColor(QColor.fromRgbF(0, 0, 0, 0))

        elif os.environ.get('XDG_CURRENT_DESKTOP') == 'KDE':
            rgb = os.popen("kreadconfig5 --group WM --key activeBackground").read().split(',')

            if len(rgb) > 1:
                r, g, b = rgb if len(rgb) == 3 else rgb[:-1]
                r = float(int(r)/255)
                g = float(int(g)/255)
                b = float(int(b)/255)
                self.web.page().setBackgroundColor(QColor.fromRgbF(r, g, b, 1))
            else:
               self.web.page().setBackgroundColor(QColor.fromRgbF(0, 0, 0, 0))

        else:
            self.web.page().setBackgroundColor(QColor.fromRgbF(0, 0, 0, 0))



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2008 Wilson Pinto Júnior <wilson@openlanhouse.org>
#  Copyright (C) 2011 Thomaz de Oliveira dos Reis <thor27@gmail.com>
#  Copyright (C) 2009  Bruno Goncalves Araujo
#  Copyright (C) 2019 Elton Fabrício Ferreira <eltonfabricio10@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import getopt

from bbv import globaldata
from bbv.server.bbv2server import run_server

class Main:
    width = 0
    height = 0
    toolkit = "auto"
    url = "/"
    window_state = None
    color = None

    def __init__(self):
        try:
            opts, args = getopt.gnu_getopt(sys.argv[1:], 'hs:vn:w:i:c:t:', ['help', 'size=', 'version', "name=",
                                                                          'window_state=', 'icon=', 'color=', 'toolkit='])
        except getopt.error as msg:
            print(msg)
            print('For help use -h or --help')
            sys.exit(2)

        if len(args):
            self.url = args[0]

        for o, a in opts:
            if o in ('-h', '--help'):
                self.help()

            elif o in ('-v', '--version'):
                print(globaldata.APP_NAME, globaldata.APP_VERSION)
                sys.exit()

            elif o in ('-s', '--size'):
                args = a.split('x')

                if len(args) != 2:
                    self.help()

                for i in args:
                    if not i.isdigit():
                        self.help()

                self.width, self.height = args

                # Window Size
                self.width = int(self.width)
                self.height = int(self.height)

            elif o in ('-t', '--toolkit'):
                if a in ("gtk", "qt"):
                    self.toolkit = a

            elif o in ('-w', '--window_state'):
                if a in ("maximized", "maximizedTop", "fullscreen", "fixed",
                         "fixedTop", "frameless", "framelessTop"):
                    self.window_state = a

            elif o in ('-i', '--icon'):
                if os.path.exists(a):
                    globaldata.ICON = a

            elif o in ('-c', '--color'):
                if a in ('black', 'none'):
                    self.color = a

            elif o in ('-n', '--name'):
                globaldata.TITLE = a

        # construct window
        if self.toolkit == "auto":
            try:
                from bbv.ui import qt
                has_qt = True
            except ImportError as e:
                print(e)
                has_qt = False

                try:
                    from bbv.ui import gtk
                    has_gtk = True
                except ImportError as e:
                    print(e)
                    has_gtk = False

            if not(has_qt) and not(has_gtk):
                print(('bbv needs WebKitGtk2 or PySide2 '
                       'to run. Please install '
                       'the latest stable version'), file=sys.stderr)
                sys.exit(1)

            elif has_qt:
                if os.environ.get('XDG_CURRENT_DESKTOP') == 'KDE':
                    os.environ['QT_QUICK_BACKEND'] = 'software'
                if os.getuid() == 0:
                    os.environ['QTWEBENGINE_DISABLE_SANDBOX'] = '1'
                self.window = qt.Window()

            elif has_gtk:
                self.window = gtk.Window()
                if globaldata.TITLE:
                    self.window.set_wmclass(globaldata.TITLE, globaldata.TITLE)

        elif self.toolkit == "gtk":
            try:
                from bbv.ui import gtk
                has_gtk = True
            except ImportError as e:
                print(e)
                has_gtk = False

            if not has_gtk:
                print(('bbv needs WebKitGtk2 '
                       'to run. Please install '
                       'the latest stable version'), file=sys.stderr)

                sys.exit(1)

            self.window = gtk.Window()
            if globaldata.TITLE:
                self.window.set_wmclass(globaldata.TITLE, globaldata.TITLE)

        elif self.toolkit == "qt":
            try:
                from bbv.ui import qt
                has_qt = True
            except ImportError as e:
                print(e)
                has_qt = False

            if not has_qt:
                print(('bbv needs PySide2 '
                       'to run. Please install '
                       'the latest stable version'), file=sys.stderr)

                sys.exit(1)

            if os.environ.get('XDG_CURRENT_DESKTOP') == 'KDE':
                os.environ['QT_QUICK_BACKEND'] = 'software'
            if os.getuid() == 0:
                os.environ['QTWEBENGINE_DISABLE_SANDBOX'] = '1'
            self.window = qt.Window()

    def help(self):
        helper = '''
  ____  _       ____            _ __      ___
 |  _ \(_)     |  _ \          | |\ \    / (_)
 | |_) |_  __ _| |_) | __ _ ___| |_\ \  / / _  _____      __
 |  _ <| |/ _` |  _ < / _` / __| '_ \ \/ / | |/ _ \ \ /\ / /
 | |_) | | (_| | |_) | (_| \__ \ | | \  /  | |  __/\ V  V /
 |____/|_|\__, |____/ \__,_|___/_| |_|\/   |_|\___| \_/\_/
           __/ |
          |___/

~$bigbashview options arguments URL
==================================================================
Options              Arguments           Descriptions
------------------------------------------------------------------
-s|--size=           widthxheight        Window size
------------------------------------------------------------------
-t|--toolkit=        gtk                 Rendering by WebKitGTK2
                     qt                  Rendering by QtWebEngine
------------------------------------------------------------------
-w|--window_state=   fullscreen          Open window in fullscreen
                     maximized           Open maximized window
                     maximizedTop        Open maximized window
                                         and keep window on top
                     fixed               Open window in fixed size
                     fixedTop            Open window in fixed size
                                         and keep window on top
                     frameless           Open window frameless
                     framelessTop        Open window frameless
                                         and keep window on top
------------------------------------------------------------------
-i|--icon=           /path/to/image      Window icon
------------------------------------------------------------------
-n|--name=           AppName             Window title
------------------------------------------------------------------
-c|--color=          black               Black background
                     none                Transparent background
------------------------------------------------------------------
-h|--help                                BigBashView help
-v|--version                             BigBashView version
==================================================================
PlainText Extension:   .txt              Text Content
------------------------------------------------------------------
HTML Extensions:       .html   |.htm     HTML Content
------------------------------------------------------------------
Executable Extensions: .sh     |.run     Shell Script
                       .sh.html|.sh.htm  Html Markup
                       .sh.php           PHP Script
                       .sh.py            Python Script
                       .sh.lua           Lua Script
                       .sh.rb            Ruby Script
                       .sh.pl            Perl Script
                       .sh.lisp          Lisp Script
                       .sh.jl            Julia Script
------------------------------------------------------------------
*Note: All executable files must have the shebang*
        '''
        print(helper)
        sys.exit()

    def run(self, start_server=True):
        server = run_server() if start_server else None

        if self.url.find('://') == -1:
            if not self.url.startswith('/'):
                self.url = '/'+self.url
            self.url = "http://%s:%s%s" % (globaldata.ADDRESS(),
                                           globaldata.PORT(), self.url)

        self.window.set_size(self.width, self.height, self.window_state)
        self.window.style(self.color)
        self.window.viewer(self.window_state)
        self.window.load_url(self.url)
        self.window.run()
        if server:
            server.stop()



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2011 Thomaz de Oliveira dos Reis <thor27@gmail.com>
#  Copyright (C) 2021 Elton Fabrício Ferreira <eltonfabricio10@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('WebKit2', '4.0')
from gi.repository import Gtk, WebKit2, Gdk

from bbv.globaldata import ICON, TITLE

class Window(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self)
        self.webview = WebKit2.WebView()
        self.webview.show()
        self.add(self.webview)
        self.set_icon_from_file(ICON)
        if TITLE:
            self.set_title(TITLE)
        else:
            self.webview.connect("notify::title", self.title_changed)
        self.webview.connect("load-changed", self.add_script)
        self.webview.connect("close", self.close_window)
        self.webview.get_settings().set_property("enable-developer-extras",True)
        self.connect("destroy", Gtk.main_quit)
        self.connect("key-press-event", self.key)

    def key(self, webview, event):
        if event.keyval == 65474:
            self.webview.reload()
        if event.keyval == 65481:
            inspector = self.webview.get_inspector()
            inspector.show()

    def add_script(self, webview, event):
        script = '''
        function _run(run){
            fetch("/execute$"+run);
        };
        '''
        if event.FINISHED:
            self.webview.run_javascript(script)

    def viewer(self, window_state):
        if window_state == "maximized":
            self.maximize()
            self.show()
        elif window_state == "fullscreen":
            self.fullscreen()
            self.show()
        elif window_state == "maximizedTop":
            self.maximize()
            self.set_keep_above(True)
            self.show()
        elif window_state == "frameless":
            self.set_decorated(False)
            self.show()
        elif window_state == "framelessTop":
            self.set_decorated(False)
            self.set_keep_above(True)
            self.show()
        elif window_state == "fixedTop":
            self.set_keep_above(True)
            self.show()
        else:
            self.show()

    def run(self):
        Gtk.main()

    def close_window(self, webview):
        Gtk.main_quit()

    def title_changed(self, webview, title):
        title = self.webview.get_title()
        os.system('''xprop -id "$(xprop -root '\t$0' _NET_ACTIVE_WINDOW | cut -f 2)" \
                    -f WM_CLASS 8s -set WM_CLASS "%s"''' % title)
        self.set_title(title)

    def load_url(self, url):
        self.webview.load_uri(url)

    def set_size(self, width, height, window_state):
        display = Gdk.Display.get_primary_monitor(Gdk.Display.get_default())
        size = display.get_geometry()
        if width <= 0:
            width = size.width/2
        if height <= 0:
            height = size.height/2

        self.set_size_request(width, height)
        self.set_position(Gtk.WindowPosition.CENTER)
        if window_state in ("fixed", "fixedTop"):
            self.set_resizable(False)

    def style(self, colorful):

        if colorful == 'black':
            self.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(0, 0, 0, 1))
            self.webview.set_background_color(Gdk.RGBA(0, 0, 0, 1))

        elif colorful == 'none':
            screen = self.get_screen()
            visual = screen.get_rgba_visual()
            if visual != None and screen.is_composited():
                self.set_visual(visual)
                self.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(0, 0, 0, 0))
                self.webview.set_background_color(Gdk.RGBA(0, 0, 0, 0))

        elif os.environ.get('XDG_CURRENT_DESKTOP') == 'KDE':
            rgb = os.popen("kreadconfig5 --group WM --key activeBackground").read().split(',')

            if len(rgb) > 1:
                r, g, b = rgb if len(rgb) == 3 else rgb[:-1]
                r = float(int(r)/255)
                g = float(int(g)/255)
                b = float(int(b)/255)
                self.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(r, g, b, 1))
                self.webview.set_background_color(Gdk.RGBA(r, g, b, 1))
            else:
                color = Gtk.Window().get_style_context().get_background_color(Gtk.StateFlags.NORMAL)
                self.override_background_color(Gtk.StateFlags.NORMAL, color)
                self.webview.set_background_color(color)

        else:
            color = Gtk.Window().get_style_context().get_background_color(Gtk.StateFlags.NORMAL)
            self.override_background_color(Gtk.StateFlags.NORMAL, color)
            self.webview.set_background_color(color)



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2011 Thomaz de Oliveira dos Reis <thor27@gmail.com>
#  Copyright (C) 2021 Elton Fabrício Ferreira <eltonfabricio10@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from urllib.parse import parse_qs
from urllib.parse import unquote
import subprocess
import os
import web
from .utils import get_env_for_shell
from .utils import to_s

from bbv import globaldata

class url_handler(object):
    __url__ = '/'

    def GET(self, name=''):
        return self.parse_and_call(web.ctx.query[1:], name)

    def POST(self, name=''):
        return self.parse_and_call(web.data(), name)

    def parse_and_call(self, qs, name):
        qs = parse_qs(qs)
        options, content = self._get_set_default_options(name)
        html = self.called(options, content, qs)
        return html

    def _get_set_default_options(self, options):
        optlist = options.split('$')
        if len(optlist) == 1:
            return ([], options)

        content = '$'.join(optlist[1:])
        optlist = optlist[0]
        if 'plain' in optlist:
            web.header('Content-Type', 'text/plain; charset=UTF-8')
        else:
            web.header('Content-Type', 'text/html; charset=UTF-8')

        return optlist, content

    def called(self, options, content, query):
        raise NotImplementedError

class content_handler(url_handler):
    __url__ = '/content(.*)'

    def called(self, options, content, query):
        try:
            with open(content) as arq:
                return arq.read()
        except UnicodeDecodeError:
            with open(content, 'rb') as arq:
                return arq.read()

class execute_handler(url_handler):
    __url__ = '/execute(.*)'

    def _execute(self, command, extra_env={}):
        env = os.environ.copy()
        env['bbv_ip'] = str(globaldata.ADDRESS())
        env['bbv_port'] = str(globaldata.PORT())
        env.update(extra_env)

        if not globaldata.ROOT_FILE:
            po = subprocess.Popen(command,
                stdin=None, stdout=subprocess.PIPE, shell=True, env=env)
        else:
            po = subprocess.Popen('/bin/bash {}'.format(command),
                stdin=None, stdout=subprocess.PIPE, shell=True, env=env)

        return po.communicate()

    def called(self, options, content, query):
        (stdout, stderr) = self._execute(
            content, extra_env=get_env_for_shell(query))
        if 'close' in options:
            os.kill(os.getpid(), 15)
        return stdout

class default_handler(url_handler):
    __url__ = '(.*)'

    def called(self, options, content, query):
        if content not in ("", "/"):
            return self.bbv_compat_mode(options, content, query)
        else:
            HTML = '''
            <html>
            <head><title>Welcome to BigBashView</title></head>
            <body style="font-size:large">
                <img src='%s'/>
                <p><i><b>
                    Hostname: <span style='color:red'> %s</span><br/>
                    Desktop Environment: <span style='color:red'> %s</span><br/>
                    Software Revision: <span style='color:red'> %s</span><br/>
                    URL:
                    <a href="#!" style='text-decoration:none'
                       onclick="_run('xdg-open https://github.com/biglinux/bigbashview')">
                       <span style='color:red'>https://github.com/biglinux/bigbashview</span>
                    </a>
                </b></i></p>
            </body>
            </html>
            ''' % (globaldata.LOGO, os.uname()[1], os.environ.get('XDG_CURRENT_DESKTOP'), globaldata.APP_VERSION)
            return HTML

    def parse_and_call(self, qs, name):
        self.original_qs = to_s(qs)
        return url_handler.parse_and_call(self, qs, name)

    def bbv_compat_mode(self, options, content, query):
        execute_ext = ('.sh','.sh.html','.sh.htm','.sh.php',
                       '.sh.py','.sh.lua','.sh.rb','.sh.pl',
                       '.sh.lisp','.sh.jl','.run')
        content_ext = ('.htm', '.html')
        content_plain_ext = ('.txt')
        content_css_ext = ('.css')
        content_js_ext = ('.js')

        relative_content = content[1:]
        if os.path.isfile(relative_content):
            if content.startswith('.'):
                content = relative_content
            else:
                content = './%s' % relative_content
        if not os.access(content, os.X_OK) and content.endswith(execute_ext):
            try:
                import stat
                os.chmod(content, os.stat(content).st_mode | stat.S_IEXEC)
            except:
                globaldata.ROOT_FILE = True
        if content.endswith(content_plain_ext):
            web.header('Content-Type', 'text/plain; charset=UTF-8')
            return content_handler().called(options, content, query)
        if content.endswith(content_css_ext):
            web.header('Content-Type', 'text/css; charset=UTF-8')
            return content_handler().called(options, content, query)
        if content.endswith(content_js_ext):
            web.header('Content-Type', 'text/javascript; charset=UTF-8')
            return content_handler().called(options, content, query)
        web.header('Content-Type', 'text/html; charset=UTF-8')
        execute_content = " ".join((content, unquote(self.original_qs)))
        if content.endswith(execute_ext):
            return execute_handler().called(options, execute_content, query)
        if content.endswith(content_ext):
            return content_handler().called(options, content, query)
        # Default option
        return content_handler().called(options, content, query)



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2011 Thomaz de Oliveira dos Reis <thor27@gmail.com>
#  Copyright (C) 2021 Elton Fabrício Ferreira <eltonfabricio10@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import web
import sys
import socket
import threading
import time
from . import views
import os

from bbv import globaldata

class Server(threading.Thread):
    def _get_subclasses(self, classes=None):
        """ Get subclasses recursively """
        if classes is None:
            classes = views.url_handler.__subclasses__()
        result = classes
        for cls in classes:
            result += self._get_subclasses(cls.__subclasses__())
        return result

    def get_urls(self):
        """ Return supported URLs. """
        classes = self._get_subclasses()
        result = []
        for cls in classes:
            result.append(cls.__url__)
            result.append(cls.__name__)
        return tuple(result)

    def get_classes(self):
        """ Return all view classes. """
        classes = self._get_subclasses()
        result = {}
        for cls in classes:
            result[cls.__name__] = cls
        return result

    def run(self):
        """ Run the webserver """
        ip = globaldata.ADDRESS()
        port = globaldata.PORT()
        sys.argv = [sys.argv[0], '']
        sys.argv[1] = ':'.join((ip, str(port)))

        urls = self.get_urls()
        classes = self.get_classes()
        self.app = web.application(urls, classes)
        self.app.run()

    def stop(self):
        print('Waiting for server to shutdown...')
        self.app.stop()
        os.kill(os.getpid(), 15)

def run_server(ip='127.0.0.1', background=True):
    soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    for port in range(19000, 19100):
        try:
            soc.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            soc.bind((ip, port))
            soc.close()
            break
        except socket.error as e:
            if e.errno != 98:
                raise socket.error(e)
            print('Port %d already in use, trying next one' % port)

    globaldata.ADDRESS = lambda: ip
    globaldata.PORT = lambda: port

    server = Server()

    if not background:
        web.config.debug = True
        return server.run()

    server.daemon = True
    web.config.debug = False
    server.start()
    # Wait for server to respond...
    while True:
        try:
            con = socket.create_connection((ip, port))
            con.close()
            break
        except socket.error as e:
            if e.errno != 111:
                raise socket.error(e)
            print('Waiting for server...')
            time.sleep(0.1)

    return server

if __name__ == "__main__":
    run_server(background=False)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

with open('README.md', 'rb') as f:
    readme = f.read().decode('utf-8')

setup(
    name='bigbashview',
    scripts=['bigbashview'],
    version='3.6.6',
    description='Graphical Frontends for shellscripts using HTML/JS/CSS',
    long_description=readme,
    long_description_content_type="text/markdown",
    author='eltonff',
    author_email='eltonfabricio10@gmail.com',
    url='https://gitlab.com/biglinux/bigbashview/tree/master/bigbashview/usr/share/bigbashview',
    install_requires=['web.py', 'PySide2'],
    license='GPLv2+',
    keywords=['big', 'bigbashview', 'biglinux', 'bash', 'webview', 'view', 'web'],
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Natural Language :: Portuguese (Brazilian)',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
)
